<?php

namespace Drupal\prometheus_metrics\Controller;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\prometheus_metrics\Bridge\PrometheusMetricsInterface;
use Drupal\prometheus_metrics\EventSubscriber\PrometheusDefaults;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Response;

/**
 * Class MetricsController.
 */
class MetricsController extends ControllerBase {
  /**
   * @var PrometheusMetricsInterface
   */
  private PrometheusMetricsInterface $prometheusMetrics;
  private string $route_memory_limit;

  public function __construct(PrometheusMetricsInterface $prometheusMetrics,ConfigFactoryInterface $configFactory){
    $this->prometheusMetrics = $prometheusMetrics;
    $config = $configFactory->get(PrometheusDefaults::CONFIGURATION_NAME);
    $this->route_memory_limit = $config->get('metrics_route_memory');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('prometheus_metrics.promphp_bridge'),
      $container->get('config.factory')
    );
  }

  /**
   * Returns all metrics collected.
   *
   * @return string
   *   Return collected metrics in Prometheus format
   */
  public function collect() {
    if(!empty($this->route_memory_limit)){
      ini_set('memory_limit', $this->route_memory_limit);
    }
    return new Response(
      $this->prometheusMetrics->render(),
      200,
      [
        'Content-Type' => 'text/plain'
      ]
    );

  }

}
