<?php

namespace Drupal\prometheus_metrics\EventSubscriber;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\prometheus_metrics\Bridge\PrometheusMetricsInterface;
use Drupal\prometheus_metrics\Events\EntityCRUDEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Counts all requests/ total requests and provides request timer metrics
 * Class PrometheusRequestSubscriber
 * @package Drupal\prometheus_metrics\EventSubscriber
 */
class PrometheusEntityCRUDSubscriber implements EventSubscriberInterface
{
  const PROMETHEUS_ENTITY_INSERT = 'entity_insert';
  const PROMETHEUS_ENTITY_UPDATE = 'entity_update';
  const PROMETHEUS_ENTITY_DELETE = 'entity_delete';

  /**
   * @var PrometheusMetricsInterface
   */
  private $prometheusMetrics;

  /**
   * @var ImmutableConfig
   */
  private $config;

  /**
   * @var string
   */
  private $namespace;

  public function __construct(
    PrometheusMetricsInterface $prometheusMetrics,
    ConfigFactoryInterface $configFactory
  ) {
    $this->prometheusMetrics = $prometheusMetrics;
    $this->config = $configFactory->get(PrometheusDefaults::CONFIGURATION_NAME);
    $this->namespace = $this->config->get('metrics_namespace')?$this->config->get('metrics_namespace'):PrometheusDefaults::METRICS_NAMESPACE;
  }

  public static function getSubscribedEvents()
  {
    return [
      self::PROMETHEUS_ENTITY_INSERT => ['onEntityInsert',0],
      self::PROMETHEUS_ENTITY_UPDATE => ['onEntityUpdate',0],
      self::PROMETHEUS_ENTITY_DELETE => ['onEntityDelete',0]
    ];
  }

  public function onEntityInsert(EntityCRUDEvent $event){
    $counter =$this->prometheusMetrics->getCounter(
      $this->namespace,
      'entity_create',
      'Counts when entity created',
      [
        'create'
      ]
    );
    $counter->inc([$event->getType()]);
  }

  public function onEntityUpdate(EntityCRUDEvent $event){
    $counter = $this->prometheusMetrics->getCounter(
      $this->namespace,
      'entity_update',
      'Counts when entity updated',
      [
        'update'
      ]
    );
    $counter->inc([$event->getType()]);
  }

  public function onEntityDelete(EntityCRUDEvent $event){
    $counter = $this->prometheusMetrics->getCounter(
      $this->namespace,
      'entity_delete',
      'Counts when entity deleted',
      [
        'delete'
      ]
    );
    $counter->inc([$event->getType()]);
  }
}
